--------------------
--01 Static Entity--
--------------------

function newStaticEntity(image_file, radius)
   local entity = {
         x  = 0, -- Coordinates
         y  = 0, 
         r  = 0, -- Rotation
         sx = 1, -- Scale
         sy = 1, 
         image    = love.graphics.newImage(image_file),
   }

   --This is so that the image will not blur when scaled up
   entity.image:setFilter('nearest', 'nearest')
   
   entity.width  = entity.image:getWidth()
   entity.height = entity.image:getHeight()
   
   -- NOTE: This is just for BASIC collision detection.
   entity.radius = radius or (entity.width / 2)
   
   entity.ox = entity.width / 2
   entity.oy = entity.height / 2
   
   function entity.update(dt)
      -- For now, this is just a filler.
   end
   
   function entity.draw()
      love.graphics.draw(entity.image, entity.x, entity.y, entity.r, entity.sx, entity.sy, entity.ox, entity.oy)
   end
   
   function entity.setPosition(x, y)
      entity.x = x
      entity.y = y
   end
  
   return entity
end

----------------------
--01 Animated Entity--
----------------------

function newAnimatedEntity(image_file, radius)
   local entity = newStaticEntity(image_file, radius) -- Yeah, right. Too lazy to type that again.
   
   -- The Basic Animation Things
   entity.animations = {}
   entity.current_animation = nil
   entity.frame_timer = 0
   entity.current_frame = 1
   entity.isEndOfFrame = false
   
   function entity.addAnimation(id, frame_rate)
      entity.animations[id] = {frame_rate = frame_rate, frames = {}}
   end
   
   -- There are 3 things added in each frame: quad, width, and height
   -- Maybe there's a quad:getWidth()?
   function entity.addAnimationFrames(id, x, y, width, height, number_of_intervals)
      -- Adding the frames can be quite unreadable, so I need these
      local image_width, image_height = entity.image:getWidth(), entity.image:getHeight()

      for i = 0, number_of_intervals - 1 do
         local start_x = x + (width * i)
         table.insert(entity.animations[id].frames,{
               quad = love.graphics.newQuad(start_x, y, width, height, image_width, image_height),
               width = width, -- I need the width and height of each frame for centering them in the .draw() function
               height = height
            }
         )
      end
   end
   
   function entity.setAnimation(id)
      if entity.current_animation ~= id then
         entity.current_animation = id
         entity.frame_timer = 0
         entity.current_frame = 1
      end
   end
   
   function entity.update(dt)
      -- Detect first if currently set animation has at least 1 frame in it
      if #entity.animations[entity.current_animation].frames > 0 then
         entity.frame_timer = entity.frame_timer + dt
         entity.isEndOfFrame = false
         -- Move to next frame according to frame rate
         if entity.frame_timer >= entity.animations[entity.current_animation].frame_rate then
            entity.frame_timer = 0
            entity.current_frame = entity.current_frame + 1
            -- Move back to first frame
            if entity.current_frame > #entity.animations[entity.current_animation].frames then
               entity.isEndOfFrame = true
               entity.current_frame = 1
            end
         end

         -- Each frame can have different dimensions
         entity.width = entity.animations[entity.current_animation].frames[entity.current_frame].width
         entity.height = entity.animations[entity.current_animation].frames[entity.current_frame].height
	          
         -- Centering the x and y to the Center of the Image
         entity.ox = entity.width / 2
         entity.oy = entity.height / 2
      end
   end
   
   function entity.draw()
      love.graphics.draw(entity.image, entity.animations[entity.current_animation].frames[entity.current_frame].quad, entity.x, entity.y, entity.r, entity.sx, entity.sy, entity.ox, entity.oy)
   end
   
   return entity
end
