function newMenu(font, button_padding)
   local menu ={ -- Ingredients of a Menu
      buttons = {}, -- A list of buttons / items
      font = font,
      button_padding = button_padding, -- How far the boxlines are to each button's text
      selected_index = nil, -- For drawing
      registered_index = nil -- So that we can cancel what button was pressed
   }
   
   function menu.addButton(text, x, y, fn) -- fn = function
      table.insert(menu.buttons, { text = text, x = x, y = y, width = 0, height = menu.font:getHeight(text), fn = fn})

      -- The following procedures center the image to the inputted x and y... renewing their x and y altogether.
      -- I just prefer using x and y at the center of the buttons.

      -- 1. Gets the width of the longest button by default and resizes each menu by that, unless you set it later on
      local width = menu.font:getWidth(text)
      for i = 1, #menu.buttons do
         if menu.buttons[i].width > width then
            width = menu.buttons[i].width
         end
      end
      
      for i = 1, #menu.buttons do
         menu.buttons[i].width = width
      end
      
      -- Doing new x and new y for the text and the box
   end
   
   -- Cycling through menu items
   function menu.cycle_up()
      if menu.selected_index == nil then
         menu.selected_index = 1
      elseif menu.selected_index == 1 then
         menu.selected_index = #menu.buttons
      else
         menu.selected_index = menu.selected_index - 1
      end
   end
   
   function menu.cycle_down()
      if menu.selected_index == nil then
         menu.selected_index = 1
      elseif menu.selected_index == #menu.buttons then
         menu.selected_index = 1
      else
         menu.selected_index = menu.selected_index + 1
      end
   end
   
   -- Detecting where the mouse is currently at (preferrably only under love.mousemoved(x, y)
   
   function menu.mouse_over(x, y)
      menu.selected_index = nil
      for i, v in pairs(menu.buttons) do
         if v.x + (v.width / 2) + menu.button_padding > x and x > v.x - (v.width / 2) - menu.button_padding then
            if v.y + (v.height / 2) + menu.button_padding > y and y > v.y - (v.height / 2) - menu.button_padding then
               menu.selected_index = i
            end
         end
      end
   end
   
   -- Pushing and Releasing Buttons: Vital for Cancelling
   
   function menu.button_press()
      menu.registered_index = menu.selected_index
   end
   
   function menu.button_release()
      local registered_index = menu.registered_index
      menu.registered_index = nil
      
      --menu.selected_index must NOT be nil for this to activate
      if menu.selected_index and menu.selected_index == registered_index then
         menu.buttons[menu.selected_index].fn()
      end
   end
   
   function menu.draw()
      love.graphics.setFont(menu.font)
      for i, v in pairs(menu.buttons) do
         -- I like using x as the center of the image instead of its true x
         local x = v.x - (v.width / 2)
         local y = v.y - (v.height / 2)
         local width = v.width + (menu.button_padding * 2)
         local height = v.height + (menu.button_padding * 2)
         
         if i == menu.registered_index then
            love.graphics.setColor(100/255, 200/255, 100/255)
            love.graphics.rectangle('fill', x - menu.button_padding, y - menu.button_padding, width, height)
            love.graphics.setColor(1, 1, 1)
         elseif i == menu.selected_index then
            love.graphics.setColor(100/255, 100/255, 100/255)
            love.graphics.rectangle('fill', x - menu.button_padding, y - menu.button_padding, width, height)
            love.graphics.setColor(1, 1, 1)
         end
         
         love.graphics.rectangle('line', x - menu.button_padding, y - menu.button_padding, width, height)
         
         x = v.x - (menu.font:getWidth(v.text) / 2)
         love.graphics.print(v.text, x, y)
      end
   end
   
   return menu
end
