state_game = state_manager.newState()

function state_game.load()
   ranch.load()
   field.load()
   shepherd = newShepherd(love.graphics.getWidth() / 2, love.graphics.getHeight() / 2)
   
   -- Release 5 Sheep
   for i = 1, 5 do
      state_game.release_sheep()
   end

   state_game.think_time = 5
   state_game.think = 0
   
   state_game.timer = newTimer(200)
   
   love.graphics.setBackgroundColor(50/255, 150/255, 50/255)
end

function state_game.update(dt)
   -- Timer is the first Priority
   state_game.timer.update(dt)
   
   if state_game.timer.ticks == 0 then
      state_manager.push(state_game_over)
   elseif #field.sheep == 0 and shepherd.state == 'free' then
      state_manager.push(state_game_success)
   end
   
   --Inputs are 2nd priority
   if love.mouse.isDown(1) and shepherd.state ~= 'lifting' and shepherd.state ~= 'dropping' then
      shepherd.setTarget(love.mouse.getX(), love.mouse.getY())
   end

   -- Release sheep from time to time
   if #ranch.sheep > 0 then
      state_game.think = state_game.think + dt
      if state_game.think >= state_game.think_time then
         state_game.rethink()
         state_game.release_sheep()
      end
   end

   -- Sheep should not be able to return to the ranch by themselves.
   for _, v in pairs(field.sheep) do
      local x1, y1 = v.x, v.y
      local x2, y2 = ranch.fence.x, ranch.fence.y
      local r1, r2 = v.radius, ranch.fence.radius
      
      if getCirclesDistance(x1, y1, r1, x2, y2, r2) < 0 then
         local a = getAngle(x2, y2, x1, y1)
         v.x, v.y = getOrbitPosition(x2, y2, a, r1 + r2)
      end
   end
   
   -- Pick up a sheep if shepherd is 'free'
   if shepherd.state == 'free' then
      local pick_up = false
      for i, v in pairs(field.sheep) do
         if not pick_up and getCirclesDistance(shepherd.x, shepherd.y, 0, v.x, v.y, v.radius) < 0 then
            pick_up = true
            shepherd.state = 'lifting'
            table.remove(field.sheep, i)
         end
      end
   elseif shepherd.state == 'lifting' then
      shepherd.isMoving = false
      shepherd.vx, shepherd.vy = 0, 0
      shepherd.setAnimation('lifting')
      if shepherd.isEndOfFrame then
         shepherd.setAnimation('carry_idle')
         shepherd.state = 'carrying'
      end
   end
   
   -- Put sheep back into ranch
   if shepherd.state == 'carrying' then
      if getCirclesDistance(shepherd.x, shepherd.y, (shepherd.radius * 2) * -1, ranch.fence.x, ranch.fence.y, ranch.fence.radius) < 0 then
         shepherd.state = 'dropping'
      end
   elseif shepherd.state == 'dropping' then
      shepherd.isMoving = false
      shepherd.vx, shepherd.vy = 0, 0
      shepherd.setAnimation('dropping')
      if shepherd.isEndOfFrame then
         shepherd.setAnimation('idle')
         shepherd.state = 'free'
         ranch.spawnsheep()
      end
   end

   ranch.update(dt)
   field.update(dt)
   
   shepherd.move(dt)
   shepherd.update(dt)
end

function state_game.draw()
   state_game.timer.draw()
   love.graphics.setFont(state_game.font)
   ranch.draw()
   field.draw()
   shepherd.draw()
end

function state_game.keypressed(key)
   if key == 'escape' then
      state_manager.push(state_pause)
   end
end

function state_game.rethink()
   state_game.think_time = love.math.random(5,15)
   state_game.think = 0
end

function state_game.release_sheep()
   local i = love.math.random(#ranch.sheep)
   local sheep = ranch.sheep[i]

   sheep.randomizeDirection()
   table.insert(field.sheep, sheep)
   table.remove(ranch.sheep, i)
end
