state_manager = {
   states = {}
}

-- This is a Stack-type State Manager

-- Notice that there is no state_manager.load(). I use state_manager.push(state) instead.
function state_manager.update(dt)
   if #state_manager.states > 0 then
      state_manager.states[#state_manager.states].update(dt)
   else
      love.event.quit() -- Let's just quit if there is nothing in the stack as to avoid errors
   end
end
function state_manager.draw()
   if #state_manager.states > 0 then
      love.graphics.push()
      state_manager.states[#state_manager.states].draw()
      love.graphics.pop()
   end
end

-- Keyboard Inputs
function state_manager.keypressed(key)
   if #state_manager.states > 0 then
      state_manager.states[#state_manager.states].keypressed(key)
   end
end
function state_manager.keyreleased(key)
   if #state_manager.states > 0 then
      state_manager.states[#state_manager.states].keyreleased(key)
   end
end
function state_manager.textinput(t)
   if #state_manager.states > 0 then
      state_manager.states[#state_manager.states].textinput(t)
   end
end

-- Mouse Functions
function state_manager.mousemoved(x, y, dx, dy)
   if #state_manager.states > 0 then
      state_manager.states[#state_manager.states].mousemoved(x, y, dx, dy)
   end
end
function state_manager.mousepressed(x, y, button)
   if #state_manager.states > 0 then
      state_manager.states[#state_manager.states].mousepressed(x, y, button)
   end
end
function state_manager.mousereleased(x, y, button)
   if #state_manager.states > 0 then
      state_manager.states[#state_manager.states].mousereleased(x, y, button)
   end
end
function state_manager.wheelmoved(x, y)
   if #state_manager.states > 0 then
      state_manager.states[#state_manager.states].wheelmoved(x, y)
   end
end

function state_manager.push(state)
   table.insert(state_manager.states, state)
   state_manager.states[#state_manager.states].load()
end
function state_manager.pop()
   table.remove(state_manager.states, #state_manager.states)
end

-----

function state_manager.newState(font)
   local state_template = { font = font or love.graphics.newFont() }
   
   function state_template.load()                      end
   function state_template.update(dt)                  end
   function state_template.draw()                      end
   function state_template.keypressed(key)             end
   function state_template.keyreleased(key)            end
   function state_template.textinput(t)                end
   function state_template.mousemoved(x, y, dx, dy)    end
   function state_template.mousepressed(x, y, button)  end
   function state_template.mousereleased(x, y, button) end
   function state_template.wheelmoved(x, y)            end
   
   return state_template
end
